import unittest
from polarion.polarion import Polarion
from keys import polarion_user, polarion_password, polarion_url
from time import sleep
import mock


class TestPolarionClient(unittest.TestCase):

    def test_wrong_url(self):
        url = polarion_url + '/wrong'
        self.assertRaises(Exception, Polarion.__init__, url,
                          polarion_user, polarion_password)

    def test_wrong_user(self):
        user = polarion_user + 'incorrect'
        self.assertRaises(Exception, Polarion.__init__, polarion_url,
                          user, polarion_password)

    def test_wrong_password(self):
        password = polarion_password + 'incorrect'
        self.assertRaises(Exception, Polarion.__init__, polarion_url,
                          polarion_user, password)

    def test_available_services_static(self):
        known_services = ['Session', 'Project', 'Tracker',
                          'Builder', 'Planning', 'TestManagement', 'Security']

        pol = Polarion(polarion_url, polarion_user,
                       polarion_password, static_service_list=True)

        for service in known_services:
            self.assertTrue(pol.hasService(service),
                            msg='Service should exist')

        self.assertFalse(pol.hasService('made_up'),
                         msg='Service should not exist')

    def test_available_services(self):
        known_services = ['Session', 'Project', 'Tracker',
                          'Builder', 'Planning', 'TestManagement', 'Security']

        pol = Polarion(polarion_url, polarion_user, polarion_password)

        for service in known_services:
            self.assertTrue(pol.hasService(service),
                            msg='Service should exist')

        self.assertFalse(pol.hasService('made_up'),
                         msg='Service should not exist')

    def test_services(self):
        known_services = ['Tracker', 'TestManagement']

        pol = Polarion(polarion_url, polarion_user, polarion_password)

        for service in known_services:
            s = pol.getService(service)
            # print(s)
            self.assertGreater(len(s.__dir__()), 10)

    def test_types(self):
        pol = Polarion(polarion_url, polarion_user, polarion_password)
        self.assertIn('EnumOptionId', str(type(pol.EnumOptionIdType)))
        self.assertIn('Text', str(type(pol.TextType)))
        self.assertIn('ArrayOfTestStepResult',
                      str(type(pol.ArrayOfTestStepResultType)))
        self.assertIn('TestStepResult', str(type(pol.TestStepResultType)))

    def test_type_wrong_service(self):
        pol = Polarion(polarion_url, polarion_user, polarion_password)

        self.assertRaises(Exception, pol.getTypeFromService, 'made_up',
                          'dont care')

    def test_string(self):
        pol = Polarion(polarion_url, polarion_user, polarion_password)

        self.assertIn(polarion_url, pol.__str__())
        self.assertIn(polarion_user, pol.__str__())
        self.assertIn(polarion_url, pol.__repr__())
        self.assertIn(polarion_user, pol.__repr__())

    @mock.patch('polarion.project.Project.__init__')
    def test_project_creation(self, mock_project):
        mock_project.return_value = None
        pol = Polarion(polarion_url, polarion_user, polarion_password)

        project = pol.getProject('Random_id')
        mock_project.assert_called_with(pol, 'Random_id')

        project = pol.getProject('other_project')
        mock_project.assert_called_with(pol, 'other_project')



from enum import Enum
from collections import namedtuple
from .factory import createFromUri
import os
import requests


class Record(object):
    """
    Create a Polarion test record,

    :param polarion: Polarion client object
    :param test_run: Test run instance
    :param polarion_record: The data from Polarion of this testrun
    :param index: The index of this record in the test run

    """
    class ResultType(Enum):
        """
        Record result enum
        """
        No = None
        PASSED = 'passed'
        FAILED = 'failed'
        BLOCKED = 'blocked'

    def __init__(self, polarion, test_run, polarion_record, index):
        self._polarion = polarion
        self._test_run = test_run
        self._polarion_record = polarion_record
        self._index = index

        self._buildWorkitemFromPolarion()

    def _buildWorkitemFromPolarion(self):
        # parse all polarion attributes to this class
        for attr, value in self._polarion_record.__dict__.items():
            for key in value:
                setattr(self, key, value[key])

        self._testcase = self._polarion_record.testCaseURI
        self._testcase_name = self._testcase.split('}')[1]
        self._defect = self._polarion_record.defectURI

    def _reloadFromPolarion(self):
        service = self._polarion.getService('TestManagement')
        self._polarion_record = service.getTestCaseRecords(self._test_run.uri, self._testcase )[0]
        self._buildWorkitemFromPolarion()
        # self._original_polarion_test_run = copy.deepcopy(self._polarion_test_run)

    def setTestStepResult(self, step_number, result: ResultType, comment=None):
        """"
        Set the result of a test step

        :param step_number: Step number
        :param result: The result fo the test step
        :param comment: An optional comment
        """
        if self.testStepResults == None:
            # get the number of test steps in
            service = self._polarion.getService('TestManagement')
            test_steps = service.getTestSteps(self.testCaseURI)
            number_of_steps = 0
            if test_steps.steps != None:
                number_of_steps = len(test_steps.steps.TestStep)
            self.testStepResults = self._polarion.ArrayOfTestStepResultType()
            for _i in range(number_of_steps):
                self.testStepResults.TestStepResult.append(
                    self._polarion.TestStepResultType())

        if step_number < len(self.testStepResults.TestStepResult):
            self.testStepResults.TestStepResult[step_number].result = self._polarion.EnumOptionIdType(
                id=result.value)
            if comment != None:
                self.testStepResults.TestStepResult[step_number].comment = self._polarion.TextType(
                    content=comment, type='text/html', contentLossy=False)

        self.save()

    def getResult(self):
        """
        Get the test result of this record

        :return: The test case result
        :rtype: ResultType
        """
        if self.result != None:
            return self.ResultType(self.result.id)
        return self.ResultType.No

    def getComment(self):
        """
        Get a comment if available. The comment may contain HTML if edited in Polarion!

        :return: Get the comment, may contain HTML
        :rtype: string
        """
        if self.comment != None:
            return self.comment.content
        return None

    @property
    def testcase_id(self):
        """
        The test case name including prefix
        """
        return self._testcase_name

    def getTestCaseName(self):
        """
        Get the test case name including prefix

        :return: The name
        :rtype: string
        """
        return self._testcase_name

    def setComment(self, comment):
        """
        tries to get the severity enum of this workitem type
        When it fails to get it, the list will be empty

        :param comment: Comment string, may contain HTML
        """
        self.comment = self._polarion.TextType(
            content=comment, type='text/html', contentLossy=False)

    def setResult(self, result: ResultType = ResultType.FAILED, comment=None):
        """
        Set the result of this record and save it.

        :param result: The result of this record
        :param comment: Comment string, may contain HTML
        """
        if comment != None:
            self.setComment(comment)
        if self.result != None:
            self.result.id = result.value
        else:
            self.result = self._polarion.EnumOptionIdType(
                id=result.value)
        self.save()

    def getExecutingUser(self):
        """
        Gets the executing user if the test was executed

        :return: The user
        :rtype: User/None
        """
        if self.executedByURI != None:
            return createFromUri(self._polarion, None, self.executedByURI)
        return None

    def hasAttachment(self):
        """
        Checks if the Record has attachments

        :return: True/False
        :rtype: boolean
        """
        if self.attachments != None:
            return True
        return False
    
    def getAttachment(self, file_name):
        """
        Get the attachment data

        :param file_name: The attachment file name
        :return: list of bytes
        :rtype: bytes[]
        """
        #find the file
        url = None
        for attachment in self.attachments.TestRunAttachment:
            if attachment.fileName == file_name:
                url = attachment.url

        if url != None:
            resp = requests.get(url, auth=(self._polarion.user, self._polarion.password))
            if resp.ok == True:
                return resp.content
            else:
                raise Exception(f'Could not download attachment {file_name}')
        else:
            raise Exception(f'Could not find attachment with name {file_name}')

    
    def saveAttachmentAsFile(self, file_name, file_path):
        """
        Save an attachment to file.

        :param file_name: The attachment file name
        :param file_path: File where to save the attachment
        """
        bin = self.getAttachment(file_name)
        with open(file_path, "wb") as file:
            file.write(bin)

    def deleteAttachment(self, file_name):
        """
        Delete an attachment.

        :param file_name: The attachment file name
        """
        service = self._polarion.getService('TestManagement')
        service.deleteAttachmentFromTestRecord(self._test_run.uri, self._index, file_name)
        self._reloadFromPolarion()

    def addAttachment(self, file_path, title):
        """
        Upload an attachment

        :param file_path: Source file to upload
        :param title: The title of the attachment
        """
        service = self._polarion.getService('TestManagement')
        file_name = os.path.split(file_path)[1]
        with open(file_path, "rb") as file_content:
            service.addAttachmentToTestRecord(self._test_run.uri, self._index, file_name, title, file_content.read())
        self._reloadFromPolarion()

    def testStepHasAttachment(self, step_index):
        """
        Checks if the a test step has attachments

        :param step_index: The test step index
        :return: True/False
        :rtype: boolean
        """
        if self.testStepResults == None:
            return False
        if self.testStepResults.TestStepResult[step_index].attachments != None:
            return True
        return False
    
    def getAttachmentFromTestStep(self, step_index, file_name):
        """
        Get the attachment data from a test step

        :param step_index: The test step index
        :param file_name: The attachment file name
        :return: list of bytes
        :rtype: bytes[]
        """
        #find the file
        url = None
        for attachment in self.testStepResults.TestStepResult[step_index].attachments.TestRunAttachment:
            if attachment.fileName == file_name:
                url = attachment.url

        if url != None:
            resp = requests.get(url, auth=(self._polarion.user, self._polarion.password))
            if resp.ok == True:
                return resp.content
            else:
                raise Exception(f'Could not download attachment {file_name}')
        else:
            raise Exception(f'Could not find attachment with name {file_name}')

    
    def saveAttachmentFromTestStepAsFile(self, step_index, file_name, file_path):
        """
        Save an attachment to file from a test step

        :param step_index: The test step index
        :param file_name: The attachment file name
        :param file_path: File where to save the attachment
        """
        bin = self.getAttachmentFromTestStep(step_index, file_name)
        with open(file_path, "wb") as file:
            file.write(bin)

    def deleteAttachmentFromTestStep(self, step_index, file_name):
        """
        Delete an attachment from a test step

        :param step_index: The test step index
        :param file_name: The attachment file name
        """
        service = self._polarion.getService('TestManagement')
        service.deleteAttachmentFromTestStep(self._test_run.uri, self._index, step_index, file_name)
        self._reloadFromPolarion()

    def addAttachmentToTestStep(self, step_index, file_path, title):
        """
        Upload an attachment to a test step

        :param step_index: The test step index
        :param file_path: Source file to upload
        :param title: The title of the attachment
        """
        service = self._polarion.getService('TestManagement')
        file_name = os.path.split(file_path)[1]
        with open(file_path, "rb") as file_content:
            service.addAttachmentToTestStep(self._test_run.uri, self._index, step_index, file_name, title, file_content.read())
        self._reloadFromPolarion()

    def save(self):
        """
        Saves the current test record
        """
        new_item = {}
        for attr, value in self.__dict__.items():
            if attr.startswith('_') != True:
                # only add if public value
                new_item[attr] = value
        service = self._polarion.getService('TestManagement')
        service.executeTest(
            self._test_run.uri, new_item)
        self._reloadFromPolarion()
        

    def __repr__(self):
        return f'{self._testcase_name} in {self._test_run.id} ({self.getResult()} on {self.executed})'

    def __str__(self):
        return f'{self._testcase_name} in {self._test_run.id} ({self.getResult()} on {self.executed})'



from zeep import Client
from zeep.plugins import HistoryPlugin
from lxml.etree import Element
from lxml import etree
import requests
import re
from urllib.parse import urljoin
from .factory import createFromUri
from .workitem import Workitem
from .testrun import Testrun
from .user import User
from .plan import Plan

class Project(object):
    """
    A Polarion project instance usable to access workitem, testruns and more. usually create by using the Polarion client.

    :param polarion: The polarion client instance
    :param project_id: The project id, as can be found in the URL of the project

    """

    def __init__(self, polarion, project_id):
        self.polarion = polarion
        self.id = project_id

        # get detaisl from polarion on this project
        service = self.polarion.getService('Project')
        try:
            self.polarion_data = service.getProject(self.id)
        except:
            raise Exception(f'Could not find project {project_id}')

        if 'name' in self.polarion_data and self.polarion_data.unresolvable == False:
            # succeeded
            self.name = self.polarion_data.name
            self.tracker_prefix = self.polarion_data.trackerPrefix
        else:
            raise Exception(f'Could not find project {project_id}')

    def getUsers(self):
        """
        Gets all users in this project
        """
        users = []
        service = self.polarion.getService('Project')
        project_users = service.getProjectUsers(self.id)
        for user in project_users:
            users.append(User(self.polarion, user))
        return users

    def findUser(self, name):
        """
        Find a specific user by id or name in this project
        """
        project_users = self.getUsers()
        for user in project_users:
            if user.id.lower() == name.lower() or user.name.lower() == name.lower():
                return user
        return None

    def getWorkitem(self, id: str):
        """Get a workitem by string

        :param id: The ID of the project workitem (PREF-123).
        :return: The request workitem
        :rtype: Workitem
        """
        return Workitem(self.polarion, self, id)

    def getPlan(self, id: str):
        """Get a plan by string

        :param id: The ID of the project plan.
        :return: The request plan
        :rtype: Plan
        """
        return Plan(self.polarion, self, id=id)

    def createPlan(self, new_plan_name, new_plan_id, new_plan_template, new_plan_parent=None):
        return Plan(self.polarion, self, new_plan_name=new_plan_name, new_plan_id=new_plan_id, new_plan_template=new_plan_template, new_plan_parent=new_plan_parent)

    def createWorkitem(self, workitem_type: str):
        return Workitem(self.polarion, self, new_workitem_type=workitem_type)

    def searchWorkitem(self, query='', order='Created', fieldList=['id'], limit=-1):
        """Query for available workitems. This will only query for the items.
        If you also want the Workitems to be retrieved, used searchWorkitemFullItem.
        
        :param query: The query to use while searching
        :param order: Order by
        :param fieldList: list of fields to retrieve for each search result
        :param limit: The limit of workitems, -1 for no limit
        :return: The search results
        :rtype: Workitem[] but only with the given fields set
        """
        query += f' AND project.id:{self.id}'
        service = self.polarion.getService('Tracker')
        return  service.queryWorkItemsLimited(
            query, order, fieldList, limit)

    def searchWorkitemFullItem(self, query='', order='Created', limit=-1):
        """Query for available workitems. This will query for the items and then fetch all result. May take a while for a big search with many results.

        :param query: The query to use while searching
        :param order: Order by
        :param limit: The limit of workitems, -1 for no limit
        :return: The search results
        :rtype: Workitem[]
        """
        return_list = []
        workitems = self.searchWorkitem(query, order, ['id'], limit)
        for workitem in workitems:
            return_list.append(
                Workitem(self.polarion, self, workitem.id))
        return return_list

    def getTestRun(self, id: str):
        """Get a testrun by string

        :param id: The ID of the project testrun.
        :return: The request testrun
        :rtype: Testrun
        """
        return Testrun(self.polarion, f'subterra:data-service:objects:/default/{self.id}${{TestRun}}{id}')

    def searchTestRuns(self, query='', order='Created', limit=-1):
        """Query for available test runs

        :param query: The query to use while searching
        :param order: Order by
        :param limit: The limit of test runs, -1 for no limit
        :return: The request testrun
        :rtype: Testrun[]
        """
        if len(query) > 0:
            query += ' AND '
        query += f'project.id:{self.id}'
        return_list = []
        service = self.polarion.getService('TestManagement')
        test_runs = service.searchTestRunsLimited(query, order, limit)
        for test_run in test_runs:
            return_list.append(
                Testrun(self.polarion, polarion_test_run=test_run))
        return return_list

    def createTestRun(self, id, title, template_id):
        """
        Create a new test run with specified title from an existing test run template
        :param id: 
        :param title: 
        :param template_id: 
        """
        service = self.polarion.getService('TestManagement')
        new_testrun_uri = service.createTestRunWithTitle(self.id, id, title, template_id)
        return createFromUri(self.polarion, self, new_testrun_uri)

    def getEnum(self, enum_name):
        """Get the options for a selected enumeration

        :param enum_name: The first part of the enum name. Will be postpended by -enum.xml by Polarion API
        :return: A list of options for the enum
        :rtype: string[]
        """
        available = []
        service = self.polarion.getService('Tracker')
        av = service.getAllEnumOptionsForId(self.id, enum_name)
        for a in av:
            if a.id not in available:
                available.append(a.id)
        return available

    def __repr__(self):
        return f'Polarion project {self.name} prefix {self.tracker_prefix}'

    def __str__(self):
        return f'Polarion project {self.name} prefix {self.tracker_prefix}'



import unittest
from polarion.polarion import Polarion
from polarion.project import Project
from polarion.user import User
from keys import polarion_user, polarion_password, polarion_url, polarion_project_id


class TestPolarionUser(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        cls.pol = Polarion(
            polarion_url, polarion_user, polarion_password)
        cls.executing_project = cls.pol.getProject(
            polarion_project_id)
            
    def setUp(self):
        pass

    def test_non_existent_user(self):
        all_users = self.executing_project.getUsers()
        broken_uri = all_users[0].uri + 'broken'

        with self.assertRaises(Exception):
            u = User(self.pol, None, broken_uri)


  
 


from .factory import Creator
from .workitem import Workitem
import copy

class Plan(object):
    """
    A polarion Plan
    """

    def __init__(self, polarion, project, polarion_record=None, uri=None, id=None, new_plan_name=None, new_plan_id=None, new_plan_parent=None, new_plan_template=None):
        """

        :param polarion: Polarion client
        :param project: Polarion project
        :param polarion_record:
        :param uri: uri for the Plan
        :param id:  id for the Plan
        :param new_plan_name: new plan name, if a new plan needs to be constructed, also supply new_plan_id and new_plan_template
        :param new_plan_id:  new plan id
        :param new_plan_parent: optional new plan parent
        :param new_plan_template: plan template, defaults in polarion are iteration or release
        """
        self._polarion = polarion
        self._project = project
        self._polarion_record = polarion_record
        self._uri = uri
        self._id = id

        if new_plan_id is not None and new_plan_name is not None:
            service = self._polarion.getService('Planning')
            self._uri = service.createPlan(self._project.id, new_plan_name, new_plan_id, new_plan_parent, new_plan_template)

        if self._uri is not None:
            service = self._polarion.getService('Planning')
            self._polarion_record = service.getPlanByUri(self._uri)

        if self._id is not None:
            service = self._polarion.getService('Planning')
            self._polarion_record = service.getPlanById(self._project.id, self._id)

        self._buildPlanFromPolarion()

    def _buildPlanFromPolarion(self):
        if self._polarion_record != None and self._polarion_record.unresolvable == False:
            # parse all polarion attributes to this class
            self._original_polarion = copy.deepcopy(self._polarion_record)
            for attr, value in self._polarion_record.__dict__.items():
                for key in value:
                    setattr(self, key, value[key])
        else:
            raise Exception(f'Plan not retrieved from Polarion')
        self._original_polarion = copy.deepcopy(self._polarion_record)

    def setDueDate(self, date):
        """
        Set the due date for this plan
        :param date: date object
        :return: None
        """
        self.dueDate = date
        self.save()

    def setStartDate(self, date):
        """
        Set the start date for this plan
        :param date: date object
        :return: None
        """
        self.startDate = date
        self.save()

    def setFinishedOnDate(self, date):
        """
        Set the finished date for this plan
        :param date: date object
        :return: None
        """
        self.finishedOn = date
        self.save()

    def setStartedOnDate(self, date):
        """
        Set the started on date for this plan
        :param date: date object
        :return: None
        """
        self.startedOn = date
        self.save()

    def addToPlan(self, workitem:Workitem):
        """
        Add a workitem to the plan
        :param workitem: Workitem
        :return: None
        """
        if any(x.id == workitem.type.id for x in self.allowedTypes.EnumOptionId):
            service = self._polarion.getService('Planning')
            service.addPlanItems(self.uri, [workitem.uri])
            workitem._reloadFromPolarion() # noqa: call private to reload from polarion so the plan status is updated
        else:
            raise Exception(f'Workitem type {workitem.id} is not allowed in this plan')

    def removeFromPlan(self, workitem: Workitem):
        """
        Remove a workitem from the plan
        :param workitem: Workitem
        :return: None
        """
        service = self._polarion.getService('Planning')
        service.removePlanItems(self.uri, [workitem.uri])
        workitem._reloadFromPolarion()  # noqa: call private to reload from polarion so the plan status is updated

    def addAllowedType(self, type):
        """
        Add an allowed workitem type to this plan
        :param type: a string with the type name
        :return: None
        """
        if any(x.id == type for x in self.allowedTypes.EnumOptionId) is False:
            service = self._polarion.getService('Planning')
            service.addPlanAllowedType(self.uri, self._polarion.EnumOptionIdType(id=type))
            self._reloadFromPolarion()

    def removeAllowedType(self, type):
        """
        Remove an allowed workitem type to this plan
        :param type: a string with the type name
        :return: None
        """
        if any(x.id == type for x in self.allowedTypes.EnumOptionId) is True:
            service = self._polarion.getService('Planning')
            service.removePlanAllowedType(self.uri, self._polarion.EnumOptionIdType(id=type))
            self._reloadFromPolarion()

    def save(self):
        """
        Update the plan in polarion
        """
        updated_plan = {}

        for attr, value in self._polarion_record.__dict__.items():
            for key in value:
                current_value = getattr(self, key)
                prev_value = getattr(self._original_polarion, key)
                if current_value != prev_value:
                    updated_plan[key] = current_value
        if len(updated_plan) > 0:
            updated_plan['uri'] = self.uri
            service = self._polarion.getService('Planning')
            service.updatePlan(updated_plan)
            self._reloadFromPolarion()

    def _reloadFromPolarion(self):
        service = self._polarion.getService('Planning')
        self._polarion_record = service.getPlanByUri(self._polarion_record.uri)
        self._buildPlanFromPolarion()
        self._original_polarion = copy.deepcopy(self._polarion_record)
        
    def __eq__(self, other):
        if self.id == other.id:
            return True
        return False

    def __repr__(self):
        return f'{self.name} ({self.id})'

    def __str__(self):
        return f'{self.name} ({self.id})'

class PlanCreator(Creator):
    def createFromUri(self, polarion, project, uri):
        return Plan(polarion, None, uri)


from .factory import Creator

class User(object):
    """
    A polarion user

    :param polarion: Polarion client object
    :param polarion_record: The user record

    """

    def __init__(self, polarion, polarion_record=None, uri=None):
        self._polarion = polarion
        self._polarion_record = polarion_record
        self._uri = uri

        if uri != None:
            service = self._polarion.getService('Project')
            self._polarion_record = service.getUserByUri(self._uri)

        if self._polarion_record!= None and self._polarion_record.unresolvable == False:
            # parse all polarion attributes to this class
            for attr, value in self._polarion_record.__dict__.items():
                for key in value:
                    setattr(self, key, value[key])
        else:
            raise Exception(f'User not retrieved from Polarion')

    def __eq__(self, other):
        if self.id == other.id:
            return True
        return False

    def __repr__(self):
        return f'{self.name} ({self.id})'

    def __str__(self):
        return f'{self.name} ({self.id})'

class UserCreator(Creator):
    def createFromUri(self, polarion, project, uri):
        return User(polarion, None, uri)


import unittest
from polarion.polarion import Polarion
from polarion.project import Project
from keys import polarion_user, polarion_password, polarion_url, polarion_project_id
from time import sleep
from datetime import datetime
import mock
from polarion.factory import createFromUri
from filecmp import cmp
from shutil import copyfile


class TestPolarionTestrun(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        cls.pol = Polarion(
            polarion_url, polarion_user, polarion_password)
        cls.executing_project = cls.pol.getProject(
            polarion_project_id)
            
        cls.executing_test_run = cls.executing_project.createTestRun('unit-' + datetime.now().strftime("%d-%m-%Y-%H-%M-%S-%f"), 'New unit test run', 'unittest-01')

        cls.checking_project = cls.pol.getProject(
            polarion_project_id)

    def setUp(self):
        pass

    def test_record_list(self):
        #build list of recods
        checking_testrun = self.checking_project.getTestRun(self.executing_test_run.id)
        list_record = []
        for r in checking_testrun.records:
            list_record.append(r.getTestCaseName())

            self.assertTrue(self.executing_test_run.hasTestCase(r.getTestCaseName()), msg='Test run does not have test case it should have')
            executing_record = self.executing_test_run.getTestCase(r.getTestCaseName())
            checking_record = checking_testrun.getTestCase(r.getTestCaseName())
            
            self.assertEqual(executing_record.getTestCaseName(), checking_record.getTestCaseName(), msg='Records did not match')
        
        self.assertFalse(self.executing_test_run.hasTestCase(list_record[0] + 'invalid'), msg='Invalid test case was available')
        self.assertIsNone(self.executing_test_run.getTestCase(list_record[0] + 'invalid'), msg='Invalid test case was returned')


    def test_run_status(self):
        self.assertEqual(self.executing_test_run.status.id, 'open', msg='Status is not open')

        self.executing_test_run.status.id = 'verifiedPassed'
        self.executing_test_run.save()

        self.assertEqual(self.executing_test_run.status.id, 'verifiedPassed', msg='Status is not verifiedPassed')

        checking_testrun = self.checking_project.getTestRun(self.executing_test_run.id)

        self.assertEqual(checking_testrun.status.id, 'verifiedPassed', msg='Status is not verifiedPassed')


  
    def test_attachment(self):
        self.assertFalse(self.executing_test_run.hasAttachment(), msg='Workitem already has attachments')
        src_1 = 'tests/test_image_1.png'
        src_2 = 'tests/test_image_2.png'
        dst = 'tests/test_image.png'
        download = 'tests/test_image_result.png'
        copyfile(src_1, dst)

        self.executing_test_run.addAttachment(dst, 'Test image 1')

        self.assertTrue(self.executing_test_run.hasAttachment(), msg='Workitem has no attachments')

        attachment_file = self.executing_test_run.attachments.TestRunAttachment[0].fileName        

        self.executing_test_run.saveAttachmentAsFile(attachment_file, download)

        self.assertTrue(cmp(src_1, download), 'File downloaded from polarion not the same')
        
        copyfile(src_2, dst)

        self.executing_test_run.updateAttachment(dst, 'Test image 1')

        self.executing_test_run.saveAttachmentAsFile(attachment_file, download)

        self.assertTrue(cmp(src_2, download), 'File downloaded from polarion not the same')
        self.assertFalse(cmp(src_1, download), 'File downloaded from polarion is the same as the old file')

        self.executing_test_run.deleteAttachment(attachment_file)

        self.assertFalse(self.executing_test_run.hasAttachment(), msg='Workitem has attachments, but should not')



from zeep import Client
from zeep.plugins import HistoryPlugin
from lxml.etree import Element
from lxml import etree
import re
from abc import ABC, abstractmethod

class Creator(ABC):
    test = 1
    @abstractmethod
    def createFromUri(self, polarion, project, uri):
        pass


creator_list = {}

def addCreator(type_name, creator):
    creator_list[type_name] = creator

def createFromUri(polarion, project, uri):
    type_name = _subterraUrl(uri)
    if type_name in creator_list:
            creator = creator_list[type_name]()
            return creator.createFromUri(polarion, project, uri)
    else:
        raise Exception(f'type {type_name} not supported')

def _subterraUrl(uri):
    uri_parts = uri.split(':')
    if uri_parts[0] != 'subterra':
        raise Exception(f'Not a subterra uri: {uri}')
    uri_type = re.findall("{(\w+)}", uri)
    if len(uri_type) == 1:
        return uri_type[0].lower()
    else:
        raise Exception(f'Not a valid polarion uri')



from zeep import Client
from zeep.plugins import HistoryPlugin
from lxml.etree import Element
from lxml import etree
import requests
import re
import copy
import os
import requests
from urllib.parse import urljoin
from .record import Record
from .factory import Creator


class Testrun(object):
    """
    Create a Polarion testrun object from uri or directly with Polarion content

    :param polarion: Polarion client object
    :param uri: Polarion uri
    :param polarion_test_run: The data from Polarion of this testrun

    :ivar records: An array of :class:`.Record`
    """

    def __init__(self, polarion, uri=None, polarion_test_run=None):
        self._polarion = polarion

        if uri != None:
            service = self._polarion.getService('TestManagement')
            try:
                self._polarion_test_run = service.getTestRunByUri(uri)
            except:
                raise Exception(f'Cannot find test run {uri}')

        elif polarion_test_run != None:
            self._polarion_test_run = polarion_test_run
        else:
            raise Exception(f'Provide either an uri or polarion_test_run ')

        self._original_polarion_test_run = copy.deepcopy(self._polarion_test_run)
        self._buildWorkitemFromPolarion()


    def _buildWorkitemFromPolarion(self):
        if self._polarion_test_run != None and self._polarion_test_run.unresolvable == False:
            for attr, value in self._polarion_test_run.__dict__.items():
                for key in value:
                    if key == 'records':
                        setattr(self, '_'+key, value[key])
                    else:
                        setattr(self, key, value[key])

            self.records = []
            self._record_dict = {}
            if self._records != None:
                # for r in self._records.TestRecord:
                for index, r in enumerate(self._records.TestRecord):
                    new_record = Record(self._polarion, self, r, index)
                    self.records.append(new_record)
                    if new_record.testcase_id not in self._record_dict:
                        self._record_dict[new_record.testcase_id] = new_record

        else:
            raise Exception(f'Testrun not retrieved from Polarion')

    def _reloadFromPolarion(self):
        service = self._polarion.getService('TestManagement')
        self._polarion_test_run = service.getTestRunByUri(self.uri)
        self._buildWorkitemFromPolarion()
        self._original_polarion_test_run = copy.deepcopy(self._polarion_test_run)

    def hasTestCase(self, id):
        """
        Checks if the the specified test case id is in the records.

        :return: True/False
        :rtype: boolean
        """
        if id in self._record_dict:
            return True
        return False

    def getTestCase(self, id):
        """
        Get the specified test case record from the test run records

        :return: Specified record fi ti exists
        :rtype: Record
        """
        if self.hasTestCase(id):
            return self._record_dict[id]
        return None

    def hasAttachment(self):
        """
        Checks if the test run has attachments

        :return: True/False
        :rtype: boolean
        """
        if self.attachments != None:
            return True
        return False
    
    def getAttachment(self, file_name):
        """
        Get the attachment data

        :param file_name: The attachment file name
        :return: list of bytes
        :rtype: bytes[]
        """
        service = self._polarion.getService('TestManagement')
        at = service.getTestRunAttachment(self.uri, file_name)
        if at != None:
            resp = requests.get(at.url, auth=(self._polarion.user, self._polarion.password))
            if resp.ok == True:
                return resp.content
        raise Exception(f'Could not download attachment {file_name}')

    
    def saveAttachmentAsFile(self, file_name, file_path):
        """
        Save an attachment to file.

        :param file_name: The attachment file name
        :param file_path: File where to save the attachment
        """
        bin = self.getAttachment(file_name)
        with open(file_path, "wb") as file:
            file.write(bin)

    def deleteAttachment(self, file_name):
        """
        Delete an attachment.

        :param file_name: The attachment file name
        """
        service = self._polarion.getService('TestManagement')
        service.deleteTestRunAttachment(self.uri, file_name)
        self._reloadFromPolarion()

    def addAttachment(self, file_path, title):
        """
        Upload an attachment

        :param file_path: Source file to upload
        :param title: The title of the attachment
        """
        service = self._polarion.getService('TestManagement')
        file_name = os.path.split(file_path)[1]
        with open(file_path, "rb") as file_content:
            service.addAttachmentToTestRun(self.uri, file_name, title, file_content.read())
        self._reloadFromPolarion()

    def updateAttachment(self, file_path, title):
        """
        Upload an attachment

        :param file_path: Source file to upload
        :param title: The title of the attachment
        """
        service = self._polarion.getService('TestManagement')
        file_name = os.path.split(file_path)[1]
        with open(file_path, "rb") as file_content:
            service.updateTestRunAttachment(self.uri, file_name, title, file_content.read())
        self._reloadFromPolarion()

    def save(self):
        """
        Update the testrun in polarion
        """
        updated_item = {}
        skip = ['records']

        for attr, value in self._polarion_test_run.__dict__.items():
            for key in value:
                if key in skip:
                    continue
                current_value = getattr(self, key)
                prev_value = getattr(self._original_polarion_test_run, key)
                if current_value != prev_value:
                    updated_item[key] = current_value
        if len(updated_item) > 0:
            updated_item['uri'] = self.uri
            service = self._polarion.getService('TestManagement')
            service.updateTestRun(updated_item)
            self._reloadFromPolarion()


    def __repr__(self):
        return f'Testrun {self.id} ({self.title}) created {self.created}'

    def __str__(self):
        return f'Testrun {self.id} ({self.title}) created {self.created}'

class TestrunCreator(Creator):
    def createFromUri(self, polarion, project, uri):
        return Testrun(polarion, uri)


from .user import UserCreator
from .testrun import TestrunCreator
from .workitem import WorkitemCreator
from .factory import addCreator

addCreator('workitem', WorkitemCreator)
addCreator('testrun', TestrunCreator)
addCreator('user', UserCreator)


from zeep import Client
from zeep.plugins import HistoryPlugin
from lxml.etree import Element
from lxml import etree
import copy
import requests
import re
import os
from urllib.parse import urljoin
from enum import Enum
from .user import User
from datetime import datetime, date
from .factory import Creator


class Workitem(object):
    """
    Create a Polarion workitem object either from and id or from an Polarion uri.

    :param polarion: Polarion client object
    :param project: Polarion Project object
    :param id: Workitem ID
    :param uri: Polarion uri

    """
    class HyperlinkRoles(Enum):
        """
        Hyperlink reference type enum
        """
        INTERNAL_REF = 'internal reference'
        EXTERNAL_REF = 'external reference'

    def __init__(self, polarion, project, id=None, uri=None, new_workitem_type=None):
        self._polarion = polarion
        self._project = project
        self._id = id
        self._uri = uri
        # new_workitem_type

        service = self._polarion.getService('Tracker')

        if self._uri:
            try:
                self._polarion_item = service.getWorkItemByUri(self._uri)
                self._id = self._polarion_item.id
            except:
                raise Exception(
                    f'Cannot find workitem {self._id} in project {self._project.id}')
        elif id != None:
            try:
                self._polarion_item = service.getWorkItemById(
                    self._project.id, self._id)
            except:
                raise Exception(
                    f'Cannot find workitem {self._id} in project {self._project.id}')
        elif new_workitem_type != None:
            self._polarion_item = self._polarion.WorkItemType(
                type=self._polarion.EnumOptionIdType(id=new_workitem_type))
            self._polarion_item.project = self._project.polarion_data
            new_uri = service.createWorkItem(self._polarion_item)
            # reload from polarion
            self._polarion_item = service.getWorkItemByUri(new_uri)
            self._id = self._polarion_item.id

        else:
            raise Exception('No id, uri of new workitem type specified!')

        self._buildWorkitemFromPolarion()

    def _buildWorkitemFromPolarion(self):
        if self._polarion_item != None and self._polarion_item.unresolvable == False:
            self._original_polarion = copy.deepcopy(self._polarion_item)
            for attr, value in self._polarion_item.__dict__.items():
                for key in value:
                    setattr(self, key, value[key])
            self._polarion_test_steps = None
            try:
                service_test = self._polarion.getService('TestManagement')
                self._polarion_test_steps = service_test.getTestSteps(self.uri)
            except:
                # fail silently as there are probably not test steps for this workitem
                # todo: logging support
                pass
            self._parsed_test_steps = None
            if self._polarion_test_steps != None:
                if self._polarion_test_steps.keys != None and self._polarion_test_steps.steps:
                    # oh god, parse the test steps...
                    columns = []
                    self._parsed_test_steps = []
                    for col in self._polarion_test_steps.keys.EnumOptionId:
                        columns.append(col.id)
                    # now parse the rows
                    for row in self._polarion_test_steps.steps.TestStep:
                        current_row = {}
                        for col_id in range(len(row.values.Text)):
                            current_row[columns[col_id]
                                        ] = row.values.Text[col_id].content
                        self._parsed_test_steps.append(current_row)
        else:
            raise Exception(f'Workitem not retrieved from Polarion')

    def getAuthor(self):
        """
        Get the author of the workitem

        :return: Author
        :rtype: User
        """
        if self.author != None:
            return User(self._polarion, self.author)
        return None

    def removeApprovee(self, user: User):
        """
        Remove a user from the approvers

        :param user: The user object to remove
        """
        service = self._polarion.getService('Tracker')
        service.removeApprovee(self.uri, user.id)
        self._reloadFromPolarion()

    def addApprovee(self, user: User, remove_others=False):
        """
        Adds a user as approvee

        :param user: The user object to add
        :param remove_others: Set to True to make the new user the only approver user.
        """
        service = self._polarion.getService('Tracker')

        if remove_others == True:
            current_users = self.getApproverUsers()
            for current_user in current_users:
                service.removeApprovee(self.uri, current_user.id)

        service.addApprovee(self.uri, user.id)
        self._reloadFromPolarion()

    def getApproverUsers(self):
        """
        Get an array of approval users

        :return: An array of User objects
        :rtype: User[]
        """
        assigned_users = []
        if self.approvals != None:
            for approval in self.approvals.Approval:
                assigned_users.append(User(self._polarion, approval.user))
        return assigned_users


    def getAssignedUsers(self):
        """
        Get an array of assigned users

        :return: An array of User objects
        :rtype: User[]
        """
        assigned_users = []
        if self.assignee != None:
            for user in self.assignee.User:
                assigned_users.append(User(self._polarion, user))
        return assigned_users

    def removeAssignee(self, user: User):
        """
        Remove a user from the assignees

        :param user: The user object to remove
        """
        service = self._polarion.getService('Tracker')
        service.removeAssignee(self.uri, user.id)
        self._reloadFromPolarion()

    def addAssignee(self, user: User, remove_others=False):
        """
        Adds a user as assignee

        :param user: The user object to add
        :param remove_others: Set to True to make the new user the only assigned user.
        """
        service = self._polarion.getService('Tracker')

        if remove_others == True:
            current_users = self.getAssignedUsers()
            for current_user in current_users:
                service.removeAssignee(self.uri, current_user.id)

        service.addAssignee(self.uri, user.id)
        self._reloadFromPolarion()

    def getStatusEnum(self):
        """
        tries to get the status enum of this workitem type
        When it fails to get it, the list will be empty

        :return: An array of strings of the statusses
        :rtype: string[]
        """
        try:
            enum = self._project.getEnum(f'{self.type.id}-status')
            return enum
        except:
            return []

    def getResolutionEnum(self):
        """
        tries to get the resolution enum of this workitem type
        When it fails to get it, the list will be empty

        :return: An array of strings of the resolutions
        :rtype: string[]
        """
        try:
            enum = self._project.getEnum(f'{self.type.id}-resolution')
            return enum
        except:
            return []

    def getSeverityEnum(self):
        """
        tries to get the severity enum of this workitem type
        When it fails to get it, the list will be empty

        :return: An array of strings of the severities
        :rtype: string[]
        """
        try:
            enum = self._project.getEnum(f'{self.type.id}-severity')
            return enum
        except:
            return []

    def getAllowedCustomKeys(self):
        """
        Gets the list of keys that the workitem is allowed to have.

        :return: An array of strings of the keys
        :rtype: string[]
        """
        try:
            service = self._polarion.getService('Tracker')
            return service.getCustomFieldKeys(self.uri)
        except:
            return []

    def getAvailableStatus(self):
        """
        Get all available status option for this workitem

        :return: An array of string of the statusses
        :rtype: string[]
        """
        available_status = []
        service = self._polarion.getService('Tracker')
        av_status = service.getAvailableEnumOptionIdsForId(self.uri, 'status')
        for status in av_status:
            available_status.append(status.id)
        return available_status

    def getAvailableActionsDetails(self):
        """
        Get all actions option for this workitem with details

        :return: An array of dictionaries of the actions
        :rtype: dict[]
        """
        available_actions = []
        service = self._polarion.getService('Tracker')
        av_actions = service.getAvailableActions(self.uri)
        for action in av_actions:
            available_actions.append(action)
        return available_actions

    def getAvailableActions(self):
        """
        Get all actions option for this workitem without details

        :return: An array of strings of the actions
        :rtype: string[]
        """
        available_actions = []
        service = self._polarion.getService('Tracker')
        av_actions = service.getAvailableActions(self.uri)
        for action in av_actions:
            available_actions.append(action.nativeActionId)
        return available_actions

    def performAction(self, action_name):
        """
        Perform selected action. An exception will be thrown if some prerequisite is not set.

        :param action_name: string containing the action name
        """
        # get id from action name
        service = self._polarion.getService('Tracker')
        av_actions = service.getAvailableActions(self.uri)
        for action in av_actions:
            if action.nativeActionId == action_name or action.actionName == action_name:
                service.performWorkflowAction(self.uri, action.actionId)

    def performActionId(self, actionId: int):
        """
        Perform selected action. An exception will be thrown if some prerequisite is not set.

        :param actionId: number for the action to perform
        """
        service = self._polarion.getService('Tracker')
        service.performWorkflowAction(self.uri, actionId)

    def setStatus(self, status):
        """
        Sets the status opf the workitem and saves the workitem, not respecting any project configured limits or requirements.

        :param status: name of the status
        """
        if status in self.getAvailableStatus():
            self.status.id = status
            self.save()

    def getDescription(self):
        """
        Get a comment if available. The comment may contain HTML if edited in Polarion!

        :return: The content of the description, may contain HTML
        :rtype: string
        """
        if self.description != None:
            return self.description.content
        return None

    def setDescription(self, description):
        """
        Sets the description and saves the workitem

        :param description: the description
        """
        self.description = self._polarion.TextType(
            content=description, type='text/html', contentLossy=False)
        self.save()

    def setResolution(self, resolution):
        """
        Sets the resolution and saves the workitem

        :param resolution: the resolution
        """

        if self.resolution != None:
            self.resolution.id = resolution
        else:
            self.resolution = self._polarion.EnumOptionIdType(
                id=resolution)
        self.save()

    def hasTestSteps(self):
        """
        Checks if the workitem has test steps

        :return: True/False
        :rtype: boolean
        """
        if self._parsed_test_steps != None:
            return len(self._parsed_test_steps) > 0
        return False

    def addComment(self, title, comment, parent=None):
        """
        Adds a comment to the workitem.

        Throws an exception if the function is disabled in Polarion.

        :param title: Title of the comment (will be None for a reply)
        :param comment: The comment, may contain html
        :param parent: A parent comment, if none provided it's a root comment.
        """
        service = self._polarion.getService('Tracker')
        if hasattr(service, 'addComment'):
            if parent == None:
                parent = self.uri
            else:
                # force title to be empty, not allowed for reply comments
                title = None
            content = {
                'type': 'text/html',
                'content': comment,
                'contentLossy': False
            }
            service.addComment(parent, title, content)
            self._reloadFromPolarion()
        else:
            raise Exception("addComment binding not found in Trackter Service. Adding comments might be disabled.")

    def setCustomField(self, key, value):
        """
        Set the custom field 'key' to the value
        :param key: custom field key
        :param value: custom field value
        :return: None
        """
        if key not in self.getAllowedCustomKeys():
            raise Exception(f"key {key} is not allowed for this workitem")
        
        if self.customFields is None:
            # nothing exists, create a custom field structure
            self.customFields = self._polarion.ArrayOfCustomType()
            self.customFields.Custom.append(self._polarion.CustomType(key=key, value=value))
        else:
            custom_field = next((custom_field for custom_field in self.customFields.Custom if custom_field["key"] == key), None)
            if custom_field is not None:
                # custom field is there and we can update the value
                custom_field.value = value
            else:
                # custom field is not there, add it.
                self.customFields.Custom.append(self._polarion.CustomType(key=key, value=value))
        self.save()

    def addHyperlink(self, url, hyperlink_type: HyperlinkRoles):
        """
        Adds a hyperlink to the workitem.

        :param url: The URL to add
        :param hyperlink_type: Select internal or external hyperlink
        """
        service = self._polarion.getService('Tracker')
        service.addHyperlink(self.uri, url, {'id': hyperlink_type.value})
        self._reloadFromPolarion()

    def addLinkedItem(self, workitem, link_type):
        """
            Add a link to a workitem

            :param workitem: A workitem
            :param link_type: The link type
        """

        service = self._polarion.getService('Tracker')
        service.addLinkedItem(self.uri, workitem.uri, role={'id': link_type})
        self._reloadFromPolarion()
        workitem._reloadFromPolarion()

    def removeLinkedItem(self, workitem, role=None):
        """
        Remove the workitem from the linked items list. If the role is specified, the specified link will be removed.
        If not specified, all links with the workitem will be removed

        :param workitem: Workitem to be removed
        :param role: the role to remove
        :return: None
        """
        service = self._polarion.getService('Tracker')
        if role is not None:
            service.removeLinkedItem(self.uri, workitem.uri, role={'id': role})
        else:
            if self.linkedWorkItems is not None:
                for linked_item in self.linkedWorkItems.LinkedWorkItem:
                    if linked_item.workItemURI == workitem.uri:
                        service.removeLinkedItem(self.uri, linked_item.workItemURI, role=linked_item.role)
            if self.linkedWorkItemsDerived is not None:
                for linked_item in self.linkedWorkItemsDerived.LinkedWorkItem:
                    if linked_item.workItemURI == workitem.uri:
                        service.removeLinkedItem(linked_item.workItemURI, self.uri, role=linked_item.role)
        self._reloadFromPolarion()
        workitem._reloadFromPolarion()

    def hasAttachment(self):
        """
        Checks if the workitem has attachments

        :return: True/False
        :rtype: boolean
        """
        if self.attachments != None:
            return True
        return False
    
    def getAttachment(self, id):
        """
        Get the attachment data

        :param id: The attachment id
        :return: list of bytes
        :rtype: bytes[]
        """
        service = self._polarion.getService('Tracker')
        return service.getAttachment(self.uri, id)

    
    def saveAttachmentAsFile(self, id, file_path):
        """
        Save an attachment to file.

        :param id: The attachment id
        :param file_path: File where to save the attachment
        """
        bin = self.getAttachment(id)
        with open(file_path, "wb") as file:
            file.write(bin)

    def deleteAttachment(self, id):
        """
        Delete an attachment.

        :param id: The attachment id
        """
        service = self._polarion.getService('Tracker')
        service.deleteAttachment(self.uri, id)
        self._reloadFromPolarion()

    def addAttachment(self, file_path, title):
        """
        Upload an attachment

        :param file_path: Source file to upload
        :param title: The title of the attachment
        """
        service = self._polarion.getService('Tracker')
        file_name = os.path.split(file_path)[1]
        with open(file_path, "rb") as file_content:
            service.createAttachment(self.uri, file_name, title, file_content.read())
        self._reloadFromPolarion()

    def updateAttachment(self, id, file_path, title):
        """
        Upload an attachment

        :param id: The attachment id
        :param file_path: Source file to upload
        :param title: The title of the attachment
        """
        service = self._polarion.getService('Tracker')
        file_name = os.path.split(file_path)[1]
        with open(file_path, "rb") as file_content:
            service.updateAttachment(self.uri, id, file_name, title, file_content.read())
        self._reloadFromPolarion()

    def save(self):
        """
        Update the workitem in polarion
        """
        updated_item = {}

        for attr, value in self._polarion_item.__dict__.items():
            for key in value:
                current_value = getattr(self, key)
                prev_value = getattr(self._original_polarion, key)
                if current_value != prev_value:
                    updated_item[key] = current_value
        if len(updated_item) > 0:
            updated_item['uri'] = self.uri
            service = self._polarion.getService('Tracker')
            service.updateWorkItem(updated_item)
            self._reloadFromPolarion()

    def _reloadFromPolarion(self):
        service = self._polarion.getService('Tracker')
        self._polarion_item = service.getWorkItemByUri(self._polarion_item.uri)
        self._buildWorkitemFromPolarion()
        self._original_polarion = copy.deepcopy(self._polarion_item)

    def __eq__(self, other):
        try:
            a = vars(self)
            b = vars(other)
        except:
            return False
        return self._compareType(a, b)

    def _compareType(self, a, b):
        basic_types = [int, float,
                       bool, type(None), str, datetime, date]

        for key in a:
            if key.startswith('_'):
                # skip private types
                continue
            # first to a quick type compare to catch any easy differences
            if type(a[key]) == type(b[key]):
                if type(a[key]) in basic_types:
                    # direct compare capable
                    if a[key] != b[key]:
                        return False
                elif isinstance(a[key], list):
                    # special case for list items
                    if len(a[key]) != len(b[key]):
                        return False
                    for idx, sub_a in enumerate(a[key]):
                        self._compareType(sub_a, b[key][idx])
                else:
                    if self._compareType(a[key], b[key]) == False:
                        return False
            else:
                # exit, type mismatch
                return False
        # survived all exits, must be good then
        return True

    def __repr__(self):
        return f'{self._id}: {self.title}'

    def __str__(self):
        return f'{self._id}: {self.title}'

class WorkitemCreator(Creator):
    def createFromUri(self, polarion, project, uri):
        return Workitem(polarion, project, None, uri)


import unittest
from polarion.polarion import Polarion
from polarion.project import Project
from polarion.record import Record
from keys import polarion_user, polarion_password, polarion_url, polarion_project_id
from time import sleep
from datetime import datetime
import mock
from polarion.factory import createFromUri
from filecmp import cmp
from shutil import copyfile


class TestPolarionTestRecord(unittest.TestCase):
    # this test requires a test run template to exist.
    # this test run template must have 3 test cases
    # all test cases must have minimum 2 steps

    @classmethod
    def setUpClass(cls):
        cls.pol = Polarion(
            polarion_url, polarion_user, polarion_password)
        cls.executing_project = cls.pol.getProject(
            polarion_project_id)
        cls.testrun_id = 'unit-' + datetime.now().strftime("%d-%m-%Y-%H-%M-%S-%f")
        cls.executing_test_run = cls.executing_project.createTestRun(cls.testrun_id, 'New unit test run', 'unittest-01')
        cls.executing_test_record = cls.executing_test_run.records[0]

        cls.checking_project = cls.pol.getProject(
            polarion_project_id)

    def setUp(self):
        pass

    def test_str_repr(self):
        self.assertIsNone(self.executing_test_run.records[2].getExecutingUser(), 'Executing use not None')

        self.assertIn(self.testrun_id, self.executing_test_run.records[2].__str__())
        self.assertIn(self.testrun_id, self.executing_test_run.records[2].__repr__())
        self.assertIn(str(Record.ResultType.No), self.executing_test_run.records[2].__str__())
        self.assertIn(str(Record.ResultType.No), self.executing_test_run.records[2].__repr__())

        self.executing_test_run.records[2].setResult(Record.ResultType.PASSED)

        self.assertIn(str(Record.ResultType.PASSED), self.executing_test_run.records[2].__str__())
        self.assertIn(str(Record.ResultType.PASSED), self.executing_test_run.records[2].__repr__())

        self.assertIn(self.pol.user, self.executing_test_run.records[2].getExecutingUser().id)



    def test_record_status(self):
        executing_test_record = self.executing_test_run.records[1]

        self.assertEqual(executing_test_record.result, None, msg='Result already set')
        self.assertEqual(executing_test_record.getResult(), Record.ResultType.No, msg='Result already set')

        passed_comment = 'This is the comment when it passed!'
        executing_test_record.setResult(Record.ResultType.PASSED, passed_comment)

        checking_test_record = self.checking_project.getTestRun(self.executing_test_run.id).records[1]

        self.assertEqual(executing_test_record.result.id, 'passed', msg='Result not set to passed')
        self.assertEqual(executing_test_record.getResult(), Record.ResultType.PASSED, msg='Result not set to passed')
        self.assertEqual(executing_test_record.getComment(), passed_comment, msg='Comment not the same')

        self.assertEqual(checking_test_record.result.id, 'passed', msg='Result not set to passed')
        self.assertEqual(checking_test_record.getResult(), Record.ResultType.PASSED, msg='Result not set to passed')
        self.assertEqual(checking_test_record.getComment(), passed_comment, msg='Comment not the same')

        new_passed_comment = 'a new comment'
        executing_test_record.setResult(Record.ResultType.BLOCKED, new_passed_comment)
        checking_test_record = self.checking_project.getTestRun(self.executing_test_run.id).records[1]

        self.assertEqual(executing_test_record.result.id, 'blocked', msg='Result not set to passed')
        self.assertEqual(executing_test_record.getResult(), Record.ResultType.BLOCKED, msg='Result not set to passed')
        self.assertIn(new_passed_comment, executing_test_record.getComment(), msg='Comment not added ')

        self.assertEqual(checking_test_record.result.id, 'blocked', msg='Result not set to passed')
        self.assertEqual(checking_test_record.getResult(), Record.ResultType.BLOCKED, msg='Result not set to passed')
        self.assertIn(new_passed_comment, checking_test_record.getComment(), msg='Comment not added')

    def test_attachment_test_step(self):
        for step_index in range(1):
            self.executing_test_record.setTestStepResult(step_index, Record.ResultType.PASSED, 'See attachment')

            self.assertFalse(self.executing_test_record.testStepHasAttachment(step_index), msg='Test step already has attachments')

            src_1 = 'tests/test_image_1.png'
            src_2 = 'tests/test_image_2.png'
            download = 'tests/test_image_result.png'

            self.executing_test_record.addAttachmentToTestStep(step_index, src_1, 'Test image 1')

            self.assertTrue(self.executing_test_record.testStepHasAttachment(step_index), msg='Test step does not have attachments')
            self.assertFalse(self.executing_test_record.testStepHasAttachment(step_index + 1), msg='Test step already has attachments')

            attachment_file1 = self.executing_test_record.testStepResults.TestStepResult[step_index].attachments.TestRunAttachment[0].fileName

            self.executing_test_record.saveAttachmentFromTestStepAsFile(step_index, attachment_file1, download)

            self.assertTrue(cmp(src_1, download), 'File downloaded from polarion not the same')
            
            self.executing_test_record.addAttachmentToTestStep(step_index, src_2, 'Test image 2')

            attachment_file2 = self.executing_test_record.testStepResults.TestStepResult[step_index].attachments.TestRunAttachment[1].fileName

            self.executing_test_record.saveAttachmentFromTestStepAsFile(step_index, attachment_file2, download)

            self.assertTrue(cmp(src_2, download), 'File downloaded from polarion not the same')
            self.assertFalse(cmp(src_1, download), 'File downloaded from polarion is the same as the old file')

            self.executing_test_record.deleteAttachmentFromTestStep(step_index, attachment_file1)
            self.executing_test_record.deleteAttachmentFromTestStep(step_index, attachment_file2)

            self.assertFalse(self.executing_test_record.testStepHasAttachment(step_index), msg='Test step still has attachments')

    def test_attachment(self):
        self.executing_test_record.setResult(Record.ResultType.BLOCKED, 'See attachment')

        self.assertFalse(self.executing_test_record.hasAttachment(), msg='Record already has attachments')
        src_1 = 'tests/test_image_1.png'
        src_2 = 'tests/test_image_2.png'
        download = 'tests/test_image_result.png'


        self.executing_test_record.addAttachment(src_1, 'Test image 1')

        self.assertTrue(self.executing_test_record.hasAttachment(), msg='Record has no attachments')

        attachment_file1 = self.executing_test_record.attachments.TestRunAttachment[0].fileName        

        self.executing_test_record.saveAttachmentAsFile(attachment_file1, download)

        self.assertTrue(cmp(src_1, download), 'File downloaded from polarion not the same')
        
        self.executing_test_record.addAttachment(src_2, 'Test image 2')

        attachment_file2 = self.executing_test_record.attachments.TestRunAttachment[1].fileName    

        self.executing_test_record.saveAttachmentAsFile(attachment_file2, download)

        self.assertTrue(cmp(src_2, download), 'File downloaded from polarion not the same')
        self.assertFalse(cmp(src_1, download), 'File downloaded from polarion is the same as the old file')

        self.executing_test_record.deleteAttachment(attachment_file1)
        self.executing_test_record.deleteAttachment(attachment_file2)

        self.assertFalse(self.executing_test_record.hasAttachment(), msg='Record has attachments, but should not')



import unittest
from polarion.polarion import Polarion
from polarion.project import Project
from keys import polarion_user, polarion_password, polarion_url, polarion_project_id
from time import sleep
import mock
from datetime import datetime


class TestPolarionProject(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        cls.pol = Polarion(polarion_url, polarion_user, polarion_password)
        cls.project = cls.pol.getProject(polarion_project_id)

    def setUp(self):
        pass

    def test_non_existing_project(self):
        # create project
        self.assertRaises(Exception, self.pol.getProject, 'fake_project')

    @mock.patch('polarion.workitem.Workitem.__init__')
    def test_get_workitem(self, mock_workitem):
        mock_workitem.return_value = None

        self.project.getWorkitem('FAKE-001')
        mock_workitem.assert_called_with(self.pol, self.project, 'FAKE-001')

        self.project.getWorkitem('FAKE-002')
        mock_workitem.assert_called_with(self.pol, self.project, 'FAKE-002')

    def test_get_all_users(self):
        all_users = self.project.getUsers()

        self.assertGreater(len(all_users), 0)

        single_user = self.project.findUser(all_users[0].id)

        self.assertIsNotNone(single_user)

    def test_get_non_existent_user(self):
        single_user = self.project.findUser(
            'kjbhjkhbk,fbdkdjsbfgd')  # no user should be found

        self.assertIsNone(single_user)

    def test_workitem_create_and_search(self):
        new_workitem = self.project.createWorkitem('task')

        self.assertIsNotNone(new_workitem)

        search_result = self.project.searchWorkitemFullItem(new_workitem.id)

        self.assertEqual(len(search_result), 1)
        self.assertEqual(new_workitem, search_result[0])

    def test_string(self):
        self.assertIn(polarion_project_id, self.project.__str__())
        self.assertIn(polarion_project_id, self.project.__repr__())

    def test_non_existent_project(self):
        self.assertRaises(Exception, Project.__init__, self.pol, 'fake_id')

    def test_non_existent_testrun(self):
        self.assertRaises(Exception, self.project.getTestRun, 'fake_id')

    def test_get_enum(self):
        status = self.project.getEnum('status')

        self.assertGreater(len(status), 0)

    def test_testrun_search(self):
        test_runs = self.project.searchTestRuns('')

        self.assertGreater(len(test_runs), 0)

        test_run = self.project.getTestRun(test_runs[0].id)
        self.assertIsNotNone(test_run)

    def test_create_testrun(self):
        test_run = self.project.createTestRun('unit-' + datetime.now().strftime("%d-%m-%Y-%H-%M-%S-%f"), 'New unit test run', 'unittest-01')
        self.assertIsNotNone(test_run)

    @mock.patch('polarion.plan.Plan.__init__')
    def test_get_plan(self, mock_plan):
        mock_plan.return_value = None

        self.project.getPlan('FAKE-001')
        mock_plan.assert_called_with(self.pol, self.project, id='FAKE-001')

        self.project.getPlan('FAKE-002')
        mock_plan.assert_called_with(self.pol, self.project, id='FAKE-002')
        



import unittest
from unittest.mock import MagicMock
from polarion.polarion import Polarion
from polarion.project import Project
from keys import polarion_user, polarion_password, polarion_url, polarion_project_id
from time import sleep
from datetime import datetime
import mock
from polarion.factory import createFromUri
from filecmp import cmp
from shutil import copyfile


class TestPolarionWorkitem(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        cls.pol = Polarion(
            polarion_url, polarion_user, polarion_password)
        cls.executing_project = cls.pol.getProject(
            polarion_project_id)
        cls.global_workitem = cls.executing_project.createWorkitem('task')

        cls.checking_project = cls.pol.getProject(
            polarion_project_id)

    def setUp(self):
        pass

    def test_create_workitem(self):
        executed_workitem = self.executing_project.createWorkitem('task')
        checking_workitem = self.checking_project.getWorkitem(
            executed_workitem.id)

        self.assertEqual(executed_workitem, checking_workitem,
                         msg='Workitems not identical')

    def test_title_change(self):
        executed_workitem = self.global_workitem
        checking_workitem = self.checking_project.getWorkitem(
            executed_workitem.id)

        # check that new title is not in item already
        new_title = 'Unit test item ' + datetime.now().strftime("%d/%m/%Y %H:%M:%S")
        self.assertNotEqual(checking_workitem.title, new_title)

        executed_workitem.title = new_title
        executed_workitem.save()
        checking_workitem = self.checking_project.getWorkitem(
            executed_workitem.id)
        self.assertEqual(executed_workitem, checking_workitem,
                         msg='Workitems not identical')

    def test_description_change(self):
        executed_workitem = self.global_workitem
        new_description = 'new description on ' + \
            datetime.now().strftime("%d/%m/%Y %H:%M:%S")

        # check that description is not the same as we're about to set
        checking_workitem = self.checking_project.getWorkitem(
            executed_workitem.id)
        self.assertNotEqual(new_description, checking_workitem.getDescription(
        ), msg='Description cannot be the same before setting')

        # update description
        executed_workitem.setDescription(new_description)

        # check that the description now is the same
        checking_workitem = self.checking_project.getWorkitem(
            executed_workitem.id)
        self.assertEqual(new_description, checking_workitem.getDescription(
        ), msg='description is not the same after setting')
        self.assertEqual(executed_workitem.getDescription(), checking_workitem.getDescription(
        ), msg='description is not the same after setting')

    def test_add_comment(self):
        executed_workitem = self.global_workitem
        comment_title = 'New comment title on ' + \
            datetime.now().strftime("%d/%m/%Y %H:%M:%S")
        comment_content = datetime.now().strftime("%d/%m/%Y %H:%M:%S")
        executed_workitem.addComment(comment_title, comment_content)

        checking_workitem = self.checking_project.getWorkitem(
            executed_workitem.id)

        found = False
        for comment in checking_workitem.comments.Comment:
            if comment.title == comment_title:
                found = True
                self.assertEqual(
                    comment.text.content, comment_content, msg='Comment content not equal')
        self.assertTrue(found, 'Comment title not found in checking workitem')

    def test_add_comment_not_available(self):
        # create a local polarion service as we'll destroy the global one
        polarion = Polarion(
            polarion_url, polarion_user, polarion_password)        

        executing_project = polarion.getProject(
            polarion_project_id)

        executing_workitem = executing_project.getWorkitem(self.global_workitem.id)

        # replace the getService with someting that will not return anything with content.
        polarion.getService = MagicMock(return_value={})

        with self.assertRaises(Exception):
            executing_workitem.addComment('', 'A comment that will fail')

    def test_assignee(self):
        self.assertEqual(len(self.global_workitem.getAssignedUsers(
        )), 0, msg='Testing workitem should have no assigned users')

        # find and add user
        user = self.executing_project.getUsers()[0]
        self.global_workitem.addAssignee(user)

        checking_workitem = self.checking_project.getWorkitem(
            self.global_workitem.id)

        executing_users = self.global_workitem.getAssignedUsers()
        checking_users = checking_workitem.getAssignedUsers()

        self.assertEqual(len(executing_users), 1,
                         msg='executing workitem should have 1 assigned users')
        self.assertEqual(len(checking_users),
                         1, msg='checking workitem should have 1 assigned users')

        # check if the users are the same
        self.assertEqual(executing_users[0], checking_users[0])

        # now remove the user
        self.global_workitem.removeAssignee(executing_users[0])

        checking_workitem = self.checking_project.getWorkitem(
            self.global_workitem.id)

        executing_users = self.global_workitem.getAssignedUsers()
        checking_users = checking_workitem.getAssignedUsers()

        self.assertEqual(len(executing_users), 0,
                         msg='executing workitem should have 0 assigned users')
        self.assertEqual(len(checking_users),
                         0, msg='checking workitem should have 0 assigned users')

    def test_replace_assignee(self):
        self.assertGreater(len(self.executing_project.getUsers(
        )), 1, msg='Project does not have more than 1 user in project')

        self.assertEqual(len(self.global_workitem.getAssignedUsers(
        )), 0, msg='Testing workitem should have no assigned users')

        # find and add user
        user = self.executing_project.getUsers()[0]

        self.global_workitem.addAssignee(user)

        checking_workitem = self.checking_project.getWorkitem(
            self.global_workitem.id)

        executing_users = self.global_workitem.getAssignedUsers()
        checking_users = checking_workitem.getAssignedUsers()

        self.assertEqual(len(executing_users), 1,
                         msg='executing workitem should have 1 assigned users')
        self.assertEqual(len(checking_users),
                         1, msg='checking workitem should have 1 assigned users')

        # check if the users are the same
        self.assertEqual(executing_users[0], checking_users[0])

        # now replace the user
        user = self.executing_project.getUsers()[1]

        self.global_workitem.addAssignee(user, True)

        checking_workitem = self.checking_project.getWorkitem(
            self.global_workitem.id)

        executing_users = self.global_workitem.getAssignedUsers()
        checking_users = checking_workitem.getAssignedUsers()

        self.assertEqual(len(executing_users), 1,
                         msg='executing workitem should have 0 assigned users')
        self.assertEqual(len(checking_users),
                         1, msg='checking workitem should have 0 assigned users')

        self.assertEqual(executing_users[0], checking_users[0])

    @mock.patch('polarion.project.Project.getEnum')
    def test_get_enum(self, mock_project):
        expected_return = ['a', 'b', 'c']
        mock_project.return_value = expected_return
        actual_return = self.global_workitem.getStatusEnum()
        mock_project.assert_called_with('task-status')
        self.assertEqual(expected_return, actual_return,
                         msg='Return value not equal to mock')

        actual_return = self.global_workitem.getResolutionEnum()
        mock_project.assert_called_with('task-resolution')
        self.assertEqual(expected_return, actual_return,
                         msg='Return value not equal to mock')

        actual_return = self.global_workitem.getSeverityEnum()
        mock_project.assert_called_with('task-severity')
        self.assertEqual(expected_return, actual_return,
                         msg='Return value not equal to mock')

        expected_return = []
        mock_project.side_effect = Exception('Dummy exception')
        actual_return = self.global_workitem.getStatusEnum()
        mock_project.assert_called_with('task-status')
        self.assertEqual(expected_return, actual_return,
                         msg='Return value not empty array')

        actual_return = self.global_workitem.getResolutionEnum()
        mock_project.assert_called_with('task-resolution')
        self.assertEqual(expected_return, actual_return,
                         msg='Return value not empty array')

        actual_return = self.global_workitem.getSeverityEnum()
        mock_project.assert_called_with('task-severity')
        self.assertEqual(expected_return, actual_return,
                         msg='Return value not empty array')

    def test_workitem_compare(self):
        executed_workitem = self.executing_project.createWorkitem('task')
        checking_workitem = self.checking_project.getWorkitem(
            executed_workitem.id)

        # status id
        new_value = 'random_val'
        executed_workitem.status.id = new_value
        self.assertNotEqual(executed_workitem, checking_workitem,
                            msg='Workitems identical, which is wrong')
        checking_workitem.status.id = new_value
        self.assertEqual(executed_workitem, checking_workitem,
                         msg='Workitems not identical')

        # title
        new_value = 'random_val'
        executed_workitem.title = new_value
        self.assertNotEqual(executed_workitem, checking_workitem,
                            msg='Workitems identical, which is wrong')
        checking_workitem.title = new_value
        self.assertEqual(executed_workitem, checking_workitem,
                         msg='Workitems not identical')

    def test_workitem_creator(self):
        new_workitem = createFromUri(self.pol, self.executing_project, self.global_workitem.uri)
        
        self.assertEqual(self.global_workitem, new_workitem,
                         msg='Workitems not identical')

    def test_workitem_author_creator(self):
        author = createFromUri(self.pol, self.executing_project, self.global_workitem.author.uri)
        
        self.assertEqual(self.global_workitem.author.id, author.id,
                         msg='Authors not identical')

    def test_attachment(self):
        self.assertFalse(self.global_workitem.hasAttachment(), msg='Workitem already has attachments')
        src_1 = 'tests/test_image_1.png'
        src_2 = 'tests/test_image_2.png'
        dst = 'tests/test_image.png'
        download = 'tests/test_image_result.png'
        copyfile(src_1, dst)

        self.global_workitem.addAttachment(dst, 'Test image 1')

        self.assertTrue(self.global_workitem.hasAttachment(), msg='Workitem has no attachments')

        attachment_id = self.global_workitem.attachments.Attachment[0].id

        self.global_workitem.saveAttachmentAsFile(attachment_id, download)

        self.assertTrue(cmp(src_1, download), 'File downloaded from polarion not the same')
        
        copyfile(src_2, dst)

        self.global_workitem.updateAttachment(attachment_id, dst, 'Test image 1')

        self.global_workitem.saveAttachmentAsFile(attachment_id, download)

        self.assertTrue(cmp(src_2, download), 'File downloaded from polarion not the same')
        self.assertFalse(cmp(src_1, download), 'File downloaded from polarion is the same as the old file')

        self.global_workitem.deleteAttachment(attachment_id)

        self.assertFalse(self.global_workitem.hasAttachment(), msg='Workitem has attachments, but should not')
    
    def test_set_status(self):
        executed_workitem = self.executing_project.createWorkitem('task')

        new_value = 'random_val'
        executed_workitem.setStatus(new_value)
        self.assertNotEqual(executed_workitem.status.id, new_value, msg="Workitem should not have updated to illegal value")

        new_value = executed_workitem.getAvailableStatus()[-1]
        executed_workitem.setStatus(new_value)
        self.assertEqual(executed_workitem.status.id, new_value, msg="Workitem should have updated to new value")

    def test_add_link(self):
        executed_workitem_1 = self.executing_project.createWorkitem('task')
        executed_workitem_2 = self.executing_project.createWorkitem('task')        

        self.assertIsNone(executed_workitem_1.linkedWorkItems, msg='Workitem already has a link')
        self.assertIsNone(executed_workitem_2.linkedWorkItems, msg='Workitem already has a link')

        executed_workitem_1.addLinkedItem(executed_workitem_2, 'relates_to')

        self.assertIsNotNone(executed_workitem_1.linkedWorkItems, msg='Workitem already has no link')

        checking_workitem_1 = self.checking_project.getWorkitem(executed_workitem_1.id)
        checking_workitem_2 = self.checking_project.getWorkitem(executed_workitem_2.id)

        self.assertEqual('relates_to', checking_workitem_1.linkedWorkItems.LinkedWorkItem[0].role.id)
        self.assertEqual('relates_to', checking_workitem_2.linkedWorkItemsDerived.LinkedWorkItem[0].role.id)
        self.assertEqual(checking_workitem_2.uri, checking_workitem_1.linkedWorkItems.LinkedWorkItem[0].workItemURI)
        self.assertEqual(checking_workitem_1.uri, checking_workitem_2.linkedWorkItemsDerived.LinkedWorkItem[0].workItemURI)


    def test_remove_link(self):
        executed_workitem_1 = self.executing_project.createWorkitem('task')
        executed_workitem_2 = self.executing_project.createWorkitem('task')      
        
        # test 1: create link and remove with role
        executed_workitem_1.addLinkedItem(executed_workitem_2, 'relates_to') 

        checking_workitem_1 = self.checking_project.getWorkitem(executed_workitem_1.id) 

        self.assertIsNotNone(executed_workitem_1.linkedWorkItems, msg='Workitem already has no link')
        self.assertIsNotNone(checking_workitem_1.linkedWorkItems, msg='Workitem already has no link')

        executed_workitem_1.removeLinkedItem(executed_workitem_2, 'relates_to')

        checking_workitem_1 = self.checking_project.getWorkitem(executed_workitem_1.id) 

        self.assertIsNone(executed_workitem_1.linkedWorkItems, msg='Workitem already has link remaining')
        self.assertIsNone(checking_workitem_1.linkedWorkItems, msg='Workitem already has link remaining')

        # test 2: create link and remove without role
        executed_workitem_1.addLinkedItem(executed_workitem_2, 'relates_to') 

        checking_workitem_1 = self.checking_project.getWorkitem(executed_workitem_1.id) 

        self.assertIsNotNone(executed_workitem_1.linkedWorkItems, msg='Workitem already has no link')
        self.assertIsNotNone(checking_workitem_1.linkedWorkItems, msg='Workitem already has no link')

        executed_workitem_1.removeLinkedItem(executed_workitem_2)

        checking_workitem_1 = self.checking_project.getWorkitem(executed_workitem_1.id) 

        self.assertIsNone(executed_workitem_1.linkedWorkItems, msg='Workitem already has link remaining')
        self.assertIsNone(checking_workitem_1.linkedWorkItems, msg='Workitem already has link remaining')

        # test 3: create link and remove from derived item        

        executed_workitem_1.addLinkedItem(executed_workitem_2, 'relates_to') 

        checking_workitem_1 = self.checking_project.getWorkitem(executed_workitem_1.id) 

        self.assertIsNotNone(executed_workitem_1.linkedWorkItems, msg='Workitem already has no link')
        self.assertIsNotNone(checking_workitem_1.linkedWorkItems, msg='Workitem already has no link')

        executed_workitem_2.removeLinkedItem(executed_workitem_1)

        checking_workitem_1 = self.checking_project.getWorkitem(executed_workitem_1.id) 

        self.assertIsNone(executed_workitem_1.linkedWorkItems, msg='Workitem already has link remaining')
        self.assertIsNone(checking_workitem_1.linkedWorkItems, msg='Workitem already has link remaining')

    def test_custom_field(self):
        executed_workitem_1 = self.executing_project.createWorkitem('task')

        self.assertIsNone(executed_workitem_1.customFields, msg='Workitem already has a custom field')

        executed_workitem_1.setCustomField(key='int_field', value=12)

        checking_workitem_1 = self.checking_project.getWorkitem(executed_workitem_1.id) 

        self.assertIsNotNone(executed_workitem_1.customFields, msg='Workitem already does not have a custom field')
        self.assertEqual(12, executed_workitem_1.customFields.Custom[0].value, msg='value not the same as set')
        self.assertIsNotNone(checking_workitem_1.customFields, msg='Workitem already does not have a custom field')
        self.assertEqual(12, checking_workitem_1.customFields.Custom[0].value, msg='value not the same as set')

        executed_workitem_1.setCustomField(key='int_field', value=24)

        checking_workitem_1 = self.checking_project.getWorkitem(executed_workitem_1.id) 

        self.assertEqual(24, executed_workitem_1.customFields.Custom[0].value, msg='value not the same as set')
        self.assertEqual(24, checking_workitem_1.customFields.Custom[0].value, msg='value not the same as set')

        executed_workitem_1.setCustomField(key='string_field', value='12')

        checking_workitem_1 = self.checking_project.getWorkitem(executed_workitem_1.id) 
        self.assertEqual('12', executed_workitem_1.customFields.Custom[0].value, msg='value not the same as set')
        self.assertEqual('12', checking_workitem_1.customFields.Custom[0].value, msg='value not the same as set')

        self.assertRaises(Exception, executed_workitem_1.setCustomField, 'random_invalid_key', 0)

    def test_approvee(self):
        executed_workitem_1 = self.executing_project.createWorkitem('task')
        all_users = self.executing_project.getUsers()
        testing_user_1 = all_users[0]
        testing_user_2 = all_users[1]

        executed_workitem_1.addApprovee(testing_user_1)

        checking_workitem_1 = self.checking_project.getWorkitem(executed_workitem_1.id) 

        self.assertEqual(executed_workitem_1.approvals.Approval[0].user.id, testing_user_1.id, msg='Approving user not equal to assign user')
        self.assertEqual(checking_workitem_1.approvals.Approval[0].user.id, testing_user_1.id, msg='Approving user not equal to assign user')

        executed_workitem_1.removeApprovee(testing_user_1)
        checking_workitem_1 = self.checking_project.getWorkitem(executed_workitem_1.id) 

        self.assertIsNone(executed_workitem_1.approvals, msg='approvee not removed')
        self.assertIsNone(checking_workitem_1.approvals, msg='approvee not removed')

        executed_workitem_1.addApprovee(testing_user_1)
        executed_workitem_1.addApprovee(testing_user_2, remove_others=True)
        checking_workitem_1 = self.checking_project.getWorkitem(executed_workitem_1.id)
        self.assertEqual(executed_workitem_1.approvals.Approval[0].user.id, testing_user_2.id, msg='Approving user not equal to assign user')
        self.assertEqual(checking_workitem_1.approvals.Approval[0].user.id, testing_user_2.id, msg='Approving user not equal to assign user')

    def test_resolution(self):
        executed_workitem_1 = self.executing_project.createWorkitem('task')

        resolutions = executed_workitem_1.getResolutionEnum()
        
        executed_workitem_1.setResolution('done')
        checking_workitem_1 = self.checking_project.getWorkitem(executed_workitem_1.id) 

        self.assertEqual(executed_workitem_1.resolution.id, 'done', msg='Resolution not as set')
        self.assertEqual(checking_workitem_1.resolution.id, 'done', msg='Resolution not as set')

        executed_workitem_1.setResolution('incomplete')
        checking_workitem_1 = self.checking_project.getWorkitem(executed_workitem_1.id) 

        self.assertEqual(executed_workitem_1.resolution.id, 'incomplete', msg='Resolution not as set')
        self.assertEqual(checking_workitem_1.resolution.id, 'incomplete', msg='Resolution not as set')



        

        





import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="polarion",
    version="0.2.0",
    author="Jesper Raemaekers",
    author_email="j.raemaekers@relek.nl",
    description="Polarion client for Python",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/jesper-raemaekers/python-polarion",
    project_urls={
        "Bug Tracker": "https://github.com/jesper-raemaekers/python-polarion/issues",
        "Documentation": "https://python-polarion.readthedocs.io/",
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Development Status :: 3 - Alpha"
    ],
    install_requires=["zeep", "lxml"],
    packages=setuptools.find_packages(),
    python_requires='>=3.6',
)



import unittest
from polarion.polarion import Polarion
from datetime import date, datetime
from keys import polarion_user, polarion_password, polarion_url, polarion_project_id


class TestPolarionPlan(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        cls.pol = Polarion(
            polarion_url, polarion_user, polarion_password)
        cls.executing_project = cls.pol.getProject(
            polarion_project_id)
        cls.executing_plan = cls.executing_project.createPlan('Test plan' + datetime.now().strftime("%d-%m-%Y-%H-%M-%S-%f"),
                                                 datetime.now().strftime("%d-%m-%Y-%H-%M-%S-%f"), 'iteration')
            
    def setUp(self):
        pass

    def test_add_remove_workitem(self):
        workitem_package = self.executing_project.createWorkitem('workpackage')

        self.executing_plan.addToPlan(workitem_package)

        self.assertEqual(workitem_package.plannedInURIs.SubterraURI[0], self.executing_plan.uri, msg='Plan in workitem does not match current plan')

        self.executing_plan.removeFromPlan(workitem_package)

        self.assertIsNone(workitem_package.plannedInURIs, msg='Workitem still in plan')

    def test_add_remove_workitem_new_type(self):
        workitem_task = self.executing_project.createWorkitem('task')

        self.assertRaises(Exception, self.executing_plan, workitem_task, msg='No exception for unsupported type')

        self.executing_plan.addAllowedType(workitem_task.type.id)  

        self.assertEqual(self.executing_plan.allowedTypes.EnumOptionId[1].id, workitem_task.type.id, msg='workitem type not added')

        self.executing_plan.addToPlan(workitem_task)

        self.assertEqual(workitem_task.plannedInURIs.SubterraURI[0], self.executing_plan.uri, msg='Plan in workitem does not match current plan')

        self.executing_plan.removeFromPlan(workitem_task)

        self.assertIsNone(workitem_task.plannedInURIs, msg='Workitem still in plan')

        self.executing_plan.removeAllowedType(workitem_task.type.id)  

        self.assertEqual(len(self.executing_plan.allowedTypes.EnumOptionId), 1, msg='still more than one allowed type')

    def test_dates(self):
        due_date = date(2021, 1, 1)
        finished_on_date = datetime(2021, 1, 2)
        start_date = date(2021, 1, 3)
        started_on_date = datetime(2021, 1, 4)

        self.executing_plan.setDueDate(due_date)
        self.executing_plan.setFinishedOnDate(finished_on_date)
        self.executing_plan.setStartDate(start_date)
        self.executing_plan.setStartedOnDate(started_on_date)

        self.checking_plan = self.executing_project.getPlan(self.executing_plan.id)        

        self.assertEqual(self.executing_plan.dueDate.day, due_date.day, msg='Date is not equal')
        self.assertEqual(self.executing_plan.finishedOn.day, finished_on_date.day, msg='Date is not equal')
        self.assertEqual(self.executing_plan.startDate.day, start_date.day, msg='Date is not equal')
        self.assertEqual(self.executing_plan.startedOn.day, started_on_date.day, msg='Date is not equal')

        self.assertEqual(self.checking_plan.dueDate.day, due_date.day, msg='Date is not equal')
        self.assertEqual(self.checking_plan.finishedOn.day, finished_on_date.day, msg='Date is not equal')
        self.assertEqual(self.checking_plan.startDate.day, start_date.day, msg='Date is not equal')
        self.assertEqual(self.checking_plan.startedOn.day, started_on_date.day, msg='Date is not equal')










  
 


from zeep import Client
from zeep.plugins import HistoryPlugin
from lxml.etree import Element
from lxml import etree
import requests
import re
from urllib.parse import urljoin
from .project import Project
import atexit

_baseServiceUrl = 'ws/services'


class Polarion(object):
    """
    Create a Polarion client which communicates to the Polarion server.

    :param polarion_url: The base URL for the polarion instance. For example http://example/com/polarion
    :param user: The user name to login
    :param password: The password for that user
    :param static_service_list: Set to True when this class may not use a request to get the services list

    """

    def __init__(self, polarion_url, user, password, static_service_list=False):
        self.user = user
        self.password = password
        self.url = polarion_url

        self.services = {}

        if not self.url.endswith('/'):
            self.url += '/'
        self.url = urljoin(self.url, _baseServiceUrl)

        if static_service_list == True:
            self._getStaticServices()
        else:
            self._getServices()
        self._createSession()
        self._getTypes()

        atexit.register(self._atexit_cleanup)
        
    def _atexit_cleanup(self):
        """
        Cleanup function to logout when Python is shutting down.
        :return: None
        """
        self.services['Session']['client'].service.endSession()

    def _getStaticServices(self):
        default_services = ['Session', 'Project', 'Tracker',
                            'Builder', 'Planning', 'TestManagement', 'Security']
        service_base_url = self.url + '/'
        for service in default_services:
            self.services[service] = {'url': urljoin(
                service_base_url, service + 'WebService')}

    def _getServices(self):
        """
        Parse the list of services available in the overview
        """
        service_overview = requests.get(self.url)
        service_base_url = self.url + '/'
        if service_overview.ok:
            services = re.findall("(\w+)WebService", service_overview.text)
            for service in services:
                if service not in self.services:
                    self.services[service] = {'url': urljoin(
                        service_base_url, service + 'WebService')}

    def _createSession(self):
        """
        Starts a session with the specified user/password
        """
        if 'Session' in self.services:
            self.history = HistoryPlugin()
            self.services['Session']['client'] = Client(
                self.services['Session']['url'] + '?wsdl', plugins=[self.history])
            try:
                self.sessionHeaderElement = None
                self.services['Session']['client'].service.logIn(
                    self.user, self.password)
                tree = self.history.last_received['envelope'].getroottree()
                self.sessionHeaderElement = tree.find(
                    './/{http://ws.polarion.com/session}sessionID')
            except:
                raise Exception(
                    f'Could not log in to Polarion for user {self.user}')
            if self.sessionHeaderElement is not None:
                self._updateServices()
        else:
            raise Exception(
                'Cannot login because WSDL has no SessionWebService')

    def _updateServices(self):
        """
        Updates all services with the correct session ID
        """
        if self.sessionHeaderElement == None:
            raise Exception('Cannot update services when not logged in')
        for service in self.services:
            if service != 'Session':
                if 'client' not in service:
                    self.services[service]['client'] = Client(
                        self.services[service]['url'] + '?wsdl')
                self.services[service]['client'].set_default_soapheaders(
                    [self.sessionHeaderElement])
            if service == 'Tracker':
                if hasattr(self.services[service]['client'].service, 'addComment'):
                    # allow addComment to be send without title, needed for reply comments
                    self.services[service]['client'].service.addComment._proxy._binding.get(
                        'addComment').input.body.type._element[1].nillable = True
            if service == 'Planning':
                self.services[service]['client'].service.createPlan._proxy._binding.get(
                        'createPlan').input.body.type._element[3].nillable = True

    def _getTypes(self):
        # TODO: check if the namespace is always the same
        self.EnumOptionIdType = self.getTypeFromService('TestManagement', 'ns3:EnumOptionId')
        self.TextType = self.getTypeFromService('TestManagement', 'ns1:Text')
        self.ArrayOfTestStepResultType = self.getTypeFromService('TestManagement', 'ns4:ArrayOfTestStepResult')
        self.TestStepResultType = self.getTypeFromService('TestManagement', 'ns4:TestStepResult')
        self.WorkItemType = self.getTypeFromService('Tracker', 'ns2:WorkItem')
        self.LinkedWorkItemType = self.getTypeFromService('Tracker', 'ns2:LinkedWorkItem')
        self.LinkedWorkItemArrayType = self.getTypeFromService('Tracker', 'ns2:ArrayOfLinkedWorkItem')
        self.ArrayOfCustomType = self.getTypeFromService('Tracker', 'ns2:ArrayOfCustom')
        self.CustomType = self.getTypeFromService('Tracker', 'ns2:Custom')
        self.ArrayOfEnumOptionIdType = self.getTypeFromService('Tracker', 'ns2:ArrayOfEnumOptionId')
        self.ArrayOfSubterraURIType = self.getTypeFromService('Tracker', 'ns1:ArrayOfSubterraURI')

    def hasService(self, name: str):
        """
        Checks if a WSDL service is available
        """
        if name in self.services:
            return True
        return False

    def getService(self, name: str):
        """
        Get a WSDL service client. The name can be 'Trakcer' or 'Session'
        """
        # request user info to see if we're still logged in
        try:
            _user = self.services['Project']['client'].service.getUser(self.user)
        except:
            # if not, create a new session
            self._createSession()

        if name in self.services:
            return self.services[name]['client'].service
        else:
            raise Exception('Service does not exsist')

    def getTypeFromService(self, name: str, type_name):
        """
        """
        if name in self.services:
            return self.services[name]['client'].get_type(type_name)
        else:
            raise Exception('Service does not exsist')

    def getProject(self, project_id):
        """Get a Polarion project

        :param project_id: The ID of the project.
        :return: The request project
        :rtype: Project
        """
        return Project(self, project_id)

    def __repr__(self):
        return f'Polarion client for {self.url} with user {self.user}'

    def __str__(self):
        return f'Polarion client for {self.url} with user {self.user}'
